@@ -1,6 +1,8 @@
 /*
  * Copyright © 2010 Codethink Limited
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -39,6 +41,10 @@
 #include "gdate.h"
 #include "genviron.h"
 
+#ifdef G_OS_UNIX
+#include "gstdio.h"
+#endif
+
 #ifdef G_OS_WIN32
 
 #define STRICT
@@ -530,16 +536,50 @@ zone_identifier_unix (void)
   gchar *canonical_path = NULL;
   GError *read_link_err = NULL;
   const gchar *tzdir;
+  gboolean not_a_symlink_to_zoneinfo = FALSE;
+  struct stat file_status;
 
   /* Resolve the actual timezone pointed to by /etc/localtime. */
   resolved_identifier = g_file_read_link ("/etc/localtime", &read_link_err);
+
+  if (resolved_identifier != NULL)
+    {
+      if (!g_path_is_absolute (resolved_identifier))
+        {
+          gchar *absolute_resolved_identifier = g_build_filename ("/etc", resolved_identifier, NULL);
+          g_free (resolved_identifier);
+          resolved_identifier = g_steal_pointer (&absolute_resolved_identifier);
+        }
+
+      if (g_lstat (resolved_identifier, &file_status) == 0)
+        {
+          if ((file_status.st_mode & S_IFMT) != S_IFREG)
+            {
+              /* Some systems (e.g. toolbox containers) make /etc/localtime be a symlink
+               * to a symlink.
+               *
+               * Rather than try to cope with that, just ignore /etc/localtime and use
+               * the fallback code to read timezone from /etc/timezone
+               */
+              g_clear_pointer (&resolved_identifier, g_free);
+              not_a_symlink_to_zoneinfo = TRUE;
+            }
+        }
+      else
+        {
+          g_clear_pointer (&resolved_identifier, g_free);
+        }
+    }
+  else
+    {
+      not_a_symlink_to_zoneinfo = g_error_matches (read_link_err,
+                                                   G_FILE_ERROR,
+                                                   G_FILE_ERROR_INVAL);
+      g_clear_error (&read_link_err);
+    }
+
   if (resolved_identifier == NULL)
     {
-      gboolean not_a_symlink = g_error_matches (read_link_err,
-                                                G_FILE_ERROR,
-                                                G_FILE_ERROR_INVAL);
-      g_clear_error (&read_link_err);
-
       /* if /etc/localtime is not a symlink, try:
        *  - /var/db/zoneinfo : 'tzsetup' program on FreeBSD and
        *    DragonflyBSD stores the timezone chosen by the user there.
@@ -549,17 +589,17 @@ zone_identifier_unix (void)
        *    as a last-ditch effort to parse the TZ= setting from within
        *    /etc/default/init
        */
-      if (not_a_symlink && (g_file_get_contents ("/var/db/zoneinfo",
-                                                 &resolved_identifier,
-                                                 NULL, NULL) ||
-                            g_file_get_contents ("/etc/timezone",
-                                                 &resolved_identifier,
-                                                 NULL, NULL)
+      if (not_a_symlink_to_zoneinfo && (g_file_get_contents ("/var/db/zoneinfo",
+                                                             &resolved_identifier,
+                                                             NULL, NULL) ||
+                                        g_file_get_contents ("/etc/timezone",
+                                                             &resolved_identifier,
+                                                             NULL, NULL)
 #if defined(__sun) && defined(__SVR4)
-                                                             ||
-                            (resolved_identifier = zone_identifier_illumos ())
+                                        ||
+                                        (resolved_identifier = zone_identifier_illumos ())
 #endif
-                                                             ))
+                                            ))
         g_strchomp (resolved_identifier);
       else
         {
@@ -2028,7 +2068,13 @@ g_time_zone_new_local (void)
  * This is equivalent to calling g_time_zone_new() with a string in the form
  * `[+|-]hh[:mm[:ss]]`.
  *
- * Returns: (transfer full): a timezone at the given offset from UTC
+ * It is possible for this function to fail if @seconds is too big (greater than
+ * 24 hours), in which case this function will return the UTC timezone for
+ * backwards compatibility. To detect failures like this, use
+ * g_time_zone_new_identifier() directly.
+ *
+ * Returns: (transfer full): a timezone at the given offset from UTC, or UTC on
+ *   failure
  * Since: 2.58
  */
 GTimeZone *
@@ -2048,11 +2094,15 @@ g_time_zone_new_offset (gint32 seconds)
                                 (ABS (seconds) / 60) % 60,
                                 ABS (seconds) % 60);
   tz = g_time_zone_new_identifier (identifier);
+
+  if (tz == NULL)
+    tz = g_time_zone_new_utc ();
+  else
+    g_assert (g_time_zone_get_offset (tz, 0) == seconds);
+
   g_assert (tz != NULL);
   g_free (identifier);
 
-  g_assert (g_time_zone_get_offset (tz, 0) == seconds);
-
   return tz;
 }
 
