@@ -1,6 +1,8 @@
 /*
  * Copyright 2020 Xavier Claessens
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -25,9 +27,6 @@ typedef struct
 static void
 test_typeof (void)
 {
-#if __cplusplus >= 201103L
-  // Test that with C++11 we don't get those kind of errors:
-  // error: invalid conversion from ‘gpointer’ {aka ‘void*’} to ‘MyObject*’ [-fpermissive]
   MyObject *obj = g_rc_box_new0 (MyObject);
   MyObject *obj2 = g_rc_box_acquire (obj);
   g_assert_true (obj2 == obj);
@@ -35,12 +34,21 @@ test_typeof (void)
   MyObject *obj3 = g_atomic_pointer_get (&obj2);
   g_assert_true (obj3 == obj);
 
+#if __cplusplus >= 201103L
   MyObject *obj4 = nullptr;
+#else
+  MyObject *obj4 = NULL;
+#endif
   g_atomic_pointer_set (&obj4, obj3);
   g_assert_true (obj4 == obj);
 
+#if __cplusplus >= 201103L
   MyObject *obj5 = nullptr;
   g_atomic_pointer_compare_and_exchange (&obj5, nullptr, obj4);
+#else
+  MyObject *obj5 = NULL;
+  g_atomic_pointer_compare_and_exchange (&obj5, NULL, obj4);
+#endif
   g_assert_true (obj5 == obj);
 
   MyObject *obj6 = g_steal_pointer (&obj5);
@@ -48,15 +56,11 @@ test_typeof (void)
 
   g_clear_pointer (&obj6, g_rc_box_release);
   g_rc_box_release (obj);
-#else
-  g_test_skip ("This test requires a C++11 compiler");
-#endif
 }
 
 static void
 test_atomic_pointer_compare_and_exchange (void)
 {
-#if __cplusplus >= 201103L
   const gchar *str1 = "str1";
   const gchar *str2 = "str2";
   const gchar *atomic_string = str1;
@@ -66,15 +70,27 @@ test_atomic_pointer_compare_and_exchange (void)
 
   g_assert_true (g_atomic_pointer_compare_and_exchange (&atomic_string, str1, str2));
   g_assert_true (atomic_string == str2);
-#else
-  g_test_skip ("This test requires a C++11 compiler");
-#endif
+}
+
+static void
+test_atomic_pointer_compare_and_exchange_full (void)
+{
+  const gchar *str1 = "str1";
+  const gchar *str2 = "str2";
+  const gchar *atomic_string = str1;
+  const gchar *old;
+
+  g_test_message ("Test that g_atomic_pointer_compare_and_exchange_full() with a "
+                  "non-void* pointer doesn’t have any compiler warnings in C++ mode");
+
+  g_assert_true (g_atomic_pointer_compare_and_exchange_full (&atomic_string, str1, str2, &old));
+  g_assert_true (atomic_string == str2);
+  g_assert_true (old == str1);
 }
 
 static void
 test_atomic_int_compare_and_exchange (void)
 {
-#if __cplusplus >= 201103L
   gint atomic_int = 5;
 
   g_test_message ("Test that g_atomic_int_compare_and_exchange() doesn’t have "
@@ -82,15 +98,25 @@ test_atomic_int_compare_and_exchange (void)
 
   g_assert_true (g_atomic_int_compare_and_exchange (&atomic_int, 5, 50));
   g_assert_cmpint (atomic_int, ==, 50);
-#else
-  g_test_skip ("This test requires a C++11 compiler");
-#endif
+}
+
+static void
+test_atomic_int_compare_and_exchange_full (void)
+{
+  gint atomic_int = 5;
+  gint old_value;
+
+  g_test_message ("Test that g_atomic_int_compare_and_exchange_full() doesn’t have "
+                  "any compiler warnings in C++ mode");
+
+  g_assert_true (g_atomic_int_compare_and_exchange_full (&atomic_int, 5, 50, &old_value));
+  g_assert_cmpint (atomic_int, ==, 50);
+  g_assert_cmpint (old_value, ==, 5);
 }
 
 static void
 test_atomic_pointer_exchange (void)
 {
-#if __cplusplus >= 201103L
   const gchar *str1 = "str1";
   const gchar *str2 = "str2";
   const gchar *atomic_string = str1;
@@ -100,24 +126,70 @@ test_atomic_pointer_exchange (void)
 
   g_assert_true (g_atomic_pointer_exchange (&atomic_string, str2) == str1);
   g_assert_true (atomic_string == str2);
-#else
-  g_test_skip ("This test requires a C++11 compiler");
-#endif
 }
 
 static void
 test_atomic_int_exchange (void)
 {
-#if __cplusplus >= 201103L
   gint atomic_int = 5;
 
   g_test_message ("Test that g_atomic_int_compare_and_exchange() doesn’t have "
                   "any compiler warnings in C++ mode");
 
   g_assert_cmpint (g_atomic_int_exchange (&atomic_int, 50), ==, 5);
-#else
-  g_test_skip ("This test requires a C++11 compiler");
-#endif
+}
+
+G_NO_INLINE
+static gboolean
+do_not_inline_this (void)
+{
+  return FALSE;
+}
+
+G_ALWAYS_INLINE
+static inline gboolean
+do_inline_this (void)
+{
+  return TRUE;
+}
+
+static void
+test_inline_no_inline_macros (void)
+{
+  g_test_message ("Test that G_NO_INLINE and G_ALWAYS_INLINE functions "
+                  "can be compiled with C++ compiler");
+
+  g_assert_false (do_not_inline_this ());
+  g_assert_true (do_inline_this ());
+}
+
+static void
+clear_boolean_ptr (gboolean *val)
+{
+  *val = TRUE;
+}
+
+static void
+test_clear_pointer (void)
+{
+  gboolean value = FALSE;
+  gboolean *ptr = &value;
+
+  g_assert_true (ptr == &value);
+  g_assert_false (value);
+  g_clear_pointer (&ptr, clear_boolean_ptr);
+  g_assert_null (ptr);
+  g_assert_true (value);
+}
+
+static void
+test_steal_pointer (void)
+{
+  gpointer ptr = &ptr;
+
+  g_assert_true (ptr == &ptr);
+  g_assert_true (g_steal_pointer (&ptr) == &ptr);
+  g_assert_null (ptr);
 }
 
 int
@@ -131,9 +203,14 @@ main (int argc, char *argv[])
 
   g_test_add_func ("/C++/typeof", test_typeof);
   g_test_add_func ("/C++/atomic-pointer-compare-and-exchange", test_atomic_pointer_compare_and_exchange);
+  g_test_add_func ("/C++/atomic-pointer-compare-and-exchange-full", test_atomic_pointer_compare_and_exchange_full);
   g_test_add_func ("/C++/atomic-int-compare-and-exchange", test_atomic_int_compare_and_exchange);
+  g_test_add_func ("/C++/atomic-int-compare-and-exchange-full", test_atomic_int_compare_and_exchange_full);
   g_test_add_func ("/C++/atomic-pointer-exchange", test_atomic_pointer_exchange);
   g_test_add_func ("/C++/atomic-int-exchange", test_atomic_int_exchange);
+  g_test_add_func ("/C++/inlined-not-inlined-functions", test_inline_no_inline_macros);
+  g_test_add_func ("/C++/clear-pointer", test_clear_pointer);
+  g_test_add_func ("/C++/steal-pointer", test_steal_pointer);
 
   return g_test_run ();
 }
